/* This is a shell created by *ChurchSkiz* used to create
a working windows program*/
/* For full functionality do not forget to add the following
files to the project and/or project folder:
winnmm.lib
ddraw.lib
ddraw.h

*/


//Includes
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
//#include <iostream.h> comment out for .net
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <io.h>
#include <fcntl.h>

//Defines
#define WINDOW_CLASS_NAME "WINCLASS1" //name of windows class we create

#define WINDOW_WIDTH	640  //starting window width
#define WINDOW_HEIGHT	480	 //starting window height

#define BALL_RADIUS 10
#define PADDLE_HEIGHT 50
#define PADDLE_WIDTH 20

#define BALL_X_SPEED 6
#define BALL_Y_SPEED 3
#define PADDLE_SPEED 4



//Macros

//used to test if keys are down or up
#define KEY_DOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEY_UP(vk_code)   ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)

//Prototypes

int Game_Init(void *parms=NULL);
int Game_Shutdown(void *parms=NULL);
int Game_Main(void *parms=NULL);



//Globals

HWND main_window_handle=NULL;  //saves window handle
HINSTANCE main_instance=NULL;  //saves the instance
char buffer[80];	//used to display text

HPEN BluePen,BlackPen,RedPen,OldPen; //Pens and brushes saved as globals because they are
//being sent to 3 functions
HBRUSH BlueBrush,RedBrush,BlackBrush,OldBrush;

RECT Ball,LeftPaddle,RightPaddle; //Rectangle for ball and paddles
int BallXPosition,BallYPosition; //x,y coordinates of ball
int dxBall,dyBall;  //velocity of ball
int dyPaddle; //velocity of paddle
int LeftPlayerScore=0, RightPlayerScore=0; //Left and Right player's score

//Functions

//the main message handler for windows
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	PAINTSTRUCT ps;
	HDC hdc;

	//bit blitting information
	HBITMAP hbmMem;
	HDC hdcMem;
	HANDLE hOld;


	//find out what message is being sent
	switch(msg)
	{
	case WM_CREATE:
		{
			//window created
			return 0;
		}break;
	case WM_PAINT:
		{
			//window needs painting
			hdc=BeginPaint(hwnd,&ps);
			EndPaint(hwnd,&ps);
			return 0;
		}break;
	case WM_DESTROY:
		{
			//program is being killed
			PostQuitMessage(0);
			return 0;
		}break;
	default:break;
	}

	//let default window handler take care of stuff we didn't
	return (DefWindowProc(hwnd,msg,wparam,lparam));
}

//WinMain
int WINAPI WinMain(HINSTANCE hinstance,HINSTANCE hprevinstance,LPSTR lpcmdline,int ncmdshow)
{
	WNDCLASS winclass;
	HWND hwnd;
	MSG msg;

	//windows class info
	winclass.style=CS_DBLCLKS|CS_OWNDC|CS_HREDRAW|CS_VREDRAW;
	winclass.lpfnWndProc=WindowProc;
	winclass.cbClsExtra=0;
	winclass.cbWndExtra=0;
	winclass.hInstance=hinstance;
	winclass.hIcon=LoadIcon(NULL, IDI_APPLICATION);
	winclass.hCursor=LoadCursor(NULL,IDC_ARROW);
	winclass.hbrBackground=(HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName=NULL;
	winclass.lpszClassName=WINDOW_CLASS_NAME;

	//register window class
	if(!RegisterClass(&winclass))
		return 0;

	//create window
	if(!(hwnd=CreateWindow(WINDOW_CLASS_NAME,"Bounce", WS_OVERLAPPED|WS_VISIBLE,
		0,0,
		WINDOW_WIDTH,WINDOW_HEIGHT,
		NULL,
		NULL,
		hinstance,
		NULL)))
		return 0;

	//save window handle and instance
	main_window_handle=hwnd;
	main_instance=hinstance;

	//perform Game initializations
	Game_Init();

	//main event loop
	while(1)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			//see if message is quit
			if (msg.message==WM_QUIT)
				break;
			
			//translate accelerator keys
			TranslateMessage(&msg);

			//send message to message processor
			DispatchMessage(&msg);
		}

		//main processing goes here
		
		Game_Main();
	}

	//shutdown program and release resources
	Game_Shutdown();

	//return to windows
	return(msg.wParam);
}

//Main Game Programming Functions

int Game_Init(void *parms)
{
	//initialization for game

	//Creations of pens and brushes (used by GDI for backgrounds and objects
	BluePen=CreatePen(PS_SOLID,0,RGB(0,0,255));
	BlackPen=CreatePen(PS_SOLID,0,RGB(0,0,0));
	RedPen=CreatePen(PS_SOLID,0,RGB(255,0,0));

	BlueBrush=CreateSolidBrush(RGB(0,0,255));
	BlackBrush=CreateSolidBrush(RGB(0,0,0));
	RedBrush=CreateSolidBrush(RGB(255,0,0));

	//Puts ball in middle of screen at start
	BallXPosition=WINDOW_WIDTH/2;
	BallYPosition=WINDOW_HEIGHT/2;

	//sets initial velocity of ball and paddles
	dxBall=BALL_X_SPEED;
	dyBall=BALL_Y_SPEED;
	dyPaddle=PADDLE_SPEED;

	//Rectangle for Ball
	Ball.left=BallXPosition-BALL_RADIUS;
	Ball.right=BallXPosition+BALL_RADIUS;
	Ball.top=BallYPosition-BALL_RADIUS;
	Ball.bottom=BallYPosition+BALL_RADIUS;

	//Rectangle for Left Paddle
	LeftPaddle.left= 2;
	LeftPaddle.right=2+PADDLE_WIDTH;
	LeftPaddle.top=WINDOW_HEIGHT/2-PADDLE_HEIGHT;
	LeftPaddle.bottom=WINDOW_HEIGHT/2+PADDLE_HEIGHT;

	//Rectangle for Right Paddle
	RightPaddle.left= WINDOW_WIDTH-8-PADDLE_WIDTH;
	RightPaddle.right=WINDOW_WIDTH-8;
	RightPaddle.top=WINDOW_HEIGHT/2-PADDLE_HEIGHT;
	RightPaddle.bottom=WINDOW_HEIGHT/2+PADDLE_HEIGHT;

	MessageBox(main_window_handle,"Welcome to Bounce!\n Brought to you by *ChurchSkiz*\nPress A and Z to control Left Player\nUp and Down arrows to control Right Player\nPress Esc to Quit","Bounce!",MB_OK);

	return 1;
}

int Game_Shutdown(void *parms)
{
	//shutdown game and release resources
	DeleteObject(BluePen);
	DeleteObject(BlackPen);
	DeleteObject(RedPen);
	DeleteObject(RedBrush);
	DeleteObject(BlueBrush);
	DeleteObject(BlackBrush);
	return 1;
}

int Game_Main(void *parms)
{
	
	//main game programming goes here
	HDC hdc;
	bool score=false;//checks to see if a goal was scored
	
	//for bit blitting
	HBITMAP hbmMem;
	HDC hdcMem;
	HANDLE hOld;
	
	hdc=GetDC(main_window_handle);

	//Input Checks
	//If Q key is pressed, game will quit
	if(KEY_DOWN(VK_ESCAPE))
	{
		PostQuitMessage(0);
	}
	//Movement W,S for left player
	if(KEY_DOWN('A'))
	{
		LeftPaddle.top-=dyPaddle;
		LeftPaddle.bottom-=dyPaddle;
	}
	if(KEY_DOWN('Z'))
	{
		LeftPaddle.top+=dyPaddle;
		LeftPaddle.bottom+=dyPaddle;
	}
	//Movement I,K for right player
	if(KEY_DOWN(VK_UP))
	{
		RightPaddle.top-=dyPaddle;
		RightPaddle.bottom-=dyPaddle;
	}
	if(KEY_DOWN(VK_DOWN))
	{
		RightPaddle.top+=dyPaddle;
		RightPaddle.bottom+=dyPaddle;
	}
	
	//Move Ball
	Ball.top+=dyBall;
	Ball.bottom+=dyBall;
	Ball.left+=dxBall;
	Ball.right+=dxBall;

	//Paddle Collisions
	//Left Paddle top and bottom wall
	if(LeftPaddle.top<=0)
	{
		LeftPaddle.top=0;
		LeftPaddle.bottom=LeftPaddle.top+PADDLE_HEIGHT*2;
	}
	if(LeftPaddle.bottom>=WINDOW_HEIGHT-32)
	{
		LeftPaddle.bottom=WINDOW_HEIGHT-32;
		LeftPaddle.top=LeftPaddle.bottom-PADDLE_HEIGHT*2;
	}
	//Right Paddle top and bottom wall
	if(RightPaddle.top<=0)
	{
		RightPaddle.top=0;
		RightPaddle.bottom=RightPaddle.top+PADDLE_HEIGHT*2;
	}
	if(RightPaddle.bottom>=WINDOW_HEIGHT-32)
	{
		RightPaddle.bottom=WINDOW_HEIGHT-32;
		RightPaddle.top=RightPaddle.bottom-PADDLE_HEIGHT*2;
	}

	
	//Bit Blitting
	//creates off-screen DC for double buffer
	hdcMem=CreateCompatibleDC(hdc);
	hbmMem=CreateCompatibleBitmap(hdc,WINDOW_WIDTH,WINDOW_HEIGHT);
	hOld=SelectObject(hdcMem,hbmMem);

	//Draw Ball
	SelectObject(hdcMem,BluePen);
	SelectObject(hdcMem,BlueBrush);
	Ellipse(hdcMem,Ball.left,Ball.top,Ball.right,Ball.bottom);
	
	//Draw Paddles
	SelectObject(hdcMem,RedPen);
	SelectObject(hdcMem,RedBrush);
	Rectangle(hdcMem,LeftPaddle.left,LeftPaddle.top,LeftPaddle.right,LeftPaddle.bottom);
	Rectangle(hdcMem,RightPaddle.left,RightPaddle.top,RightPaddle.right,RightPaddle.bottom);

	//Transfer off-screen to primary
	BitBlt(hdc,0,0,WINDOW_WIDTH,WINDOW_HEIGHT,hdcMem,0,0,SRCCOPY);

	//Free up offscreen DC
	SelectObject(hdcMem,hOld);
	DeleteObject(hbmMem);
	DeleteDC(hdcMem);

	//Ball Collisions
	//Checks against right and left wall
	//right wall, left player scores
	if(Ball.right>=WINDOW_WIDTH-7)//7 is a static number see comment a few lines down
	{
		LeftPlayerScore++;
		dxBall=-dxBall;
		//reset ball position
		Ball.left=BallXPosition-BALL_RADIUS;
		Ball.right=BallXPosition+BALL_RADIUS;
		Ball.top=BallYPosition-BALL_RADIUS;
		Ball.bottom=BallYPosition+BALL_RADIUS;
		//display message of score
		sprintf(buffer,"Score is %d to %d!",LeftPlayerScore,RightPlayerScore);
		MessageBox(main_window_handle,buffer,"GOAL!",MB_OK);
	}
	//left wall, right player scores
	if(Ball.left<=0)
	{
		RightPlayerScore++;
		dxBall=-dxBall;
		//reset ball position
		Ball.left=BallXPosition-BALL_RADIUS;
		Ball.right=BallXPosition+BALL_RADIUS;
		Ball.top=BallYPosition-BALL_RADIUS;
		Ball.bottom=BallYPosition+BALL_RADIUS;
		//display message of score
		sprintf(buffer,"Score is %d to %d!",LeftPlayerScore,RightPlayerScore);
		MessageBox(main_window_handle,buffer,"GOAL!",MB_OK);	
	}

	//Checks against Top and Bottom wall
	if(Ball.top<=0)
	{dyBall=-dyBall;}
	if(Ball.bottom>=WINDOW_HEIGHT-32)//32 is a static number to take into account the client
		//area versus total window height (menu bar takes up some space)
	{dyBall=-dyBall;}
	//Check against Left Paddle
	//if ball is under top of left paddle and over bottom of left paddle
	//and ball is hitting side of paddle
	if(((Ball.top>=LeftPaddle.top && Ball.top<=LeftPaddle.bottom)||(Ball.bottom>=LeftPaddle.top && Ball.bottom<=LeftPaddle.bottom)) && Ball.left==LeftPaddle.right)
	{
		dxBall=-dxBall;
	}
	//check against top and bottom of left paddle
	//if ball is left of paddle, bottom of ball is below top and top of ball is above top 
	if(Ball.left<LeftPaddle.right && Ball.bottom >= LeftPaddle.top && Ball.top <= LeftPaddle.top)
	{
		dyBall=-dyBall;
	}
	if(Ball.left<LeftPaddle.right && Ball.top <=LeftPaddle.bottom && Ball.bottom >= LeftPaddle.bottom)
	{
		dyBall=-dyBall;
	}
	//Check against Right Paddle
	if(((Ball.top>=RightPaddle.top && Ball.top<=RightPaddle.bottom)||(Ball.bottom>=RightPaddle.top && Ball.bottom<=RightPaddle.bottom)) && Ball.right==RightPaddle.left)
	{
		dxBall=-dxBall;
	}
	//check against top and bottom of right paddle
	if(Ball.right>RightPaddle.left && Ball.bottom >=LeftPaddle.top && Ball.top <= LeftPaddle.top)
	{
		dyBall=-dyBall;
	}
	if(Ball.right>RightPaddle.left && Ball.top <=RightPaddle.bottom && Ball.bottom >=RightPaddle.bottom)
	{
		dyBall=-dyBall;
	}
	//Releases control of GDC
	ReleaseDC(main_window_handle,hdc);

	//Timing
	Sleep(1);
	return 1;
}

//Additional Game Functions